/*============================================================================
  File:     Indexing for Aggregates.sql

  Summary:  This script sets up the queries and indexes needed to show performance
			gains for indexing aggregate queries.

			If you need credit you can download and unzip to restore this
			SQL Server 2000 backup.
			http://www.sqlskills.com/resources/conferences/CreditBackup80.zip

  Date:     November 2005

  Version:	SQL Server 2005 RTM
------------------------------------------------------------------------------
  Copyright (C) 2005 Kimberly L. Tripp, SYSolutions, Inc.
  All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

SET STATISTICS IO ON
-- Turn Graphical Showplan ON (Ctrl+K)

USE CREDIT
go

--Original Query
SELECT member_no AS MemberNo, 
	sum(charge_amt) AS TotalSales
FROM dbo.charge 	-- with no useful indexes this does a table scan 
					-- and hash aggregate
GROUP BY member_no
-- Run this once withOUT the order by... HASH aggregates do NOT return
-- the data ordered. You MUST add order by if you want the data ordered.
ORDER BY member_no -- you probably want the data ordered?
go

CREATE INDEX Covering1 on dbo.charge(charge_amt, member_no) 
	-- Not in order by the Group By 
go

-- Run the query again - you'll see HASH AGGREGATE but on a narrower
-- set... When you compare these you'll see that this is faster.
go

SELECT member_no AS MemberNo, 
	sum(charge_amt) AS TotalSales
FROM dbo.charge
GROUP BY member_no
ORDER BY member_no -- TO be fair in the comparison you should all the ORDER BY
go

CREATE INDEX Covering2 
ON charge(member_no, charge_amt) -- In Order by the Group By 
go
-- Ideal case for an INCLUDE index
CREATE INDEX CoveringWithInclude 
ON dbo.charge (member_no)
INCLUDE (charge_amt)
go		

-- Run the query again - you'll see STREAM
SELECT member_no AS MemberNo, 
	sum(charge_amt) AS TotalSales
FROM dbo.charge WITH (INDEX(Covering2))
GROUP BY member_no
ORDER BY member_no -- TO be fair in the comparison you should all the ORDER BY
						-- BUT because this is a stream aggregate you'll see NO
						-- additional costs are added to the aggregate.
go
SELECT member_no AS MemberNo, 
	sum(charge_amt) AS TotalSales
FROM dbo.charge WITH (INDEX(CoveringWithInclude))
GROUP BY member_no
ORDER BY member_no

-- DROP ALL Indexes Added for this Demo
-- DROP INDEX Charge.Covering1
-- DROP INDEX Charge.Covering2
-- DROP INDEX Charge.CoveringWithInclude